﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace LovciPokladu
{
    public class Floodfill
    {
        public int[,] Sit;
        public List<Tuple<int, FFNode>> Cesty = new List<Tuple<int, FFNode>>();

        public FFNode NejlepsiCesta { get { return Cesty.OrderByDescending(c => c.Item1).First().Item2; } }
        public Tuple<int, int> DalsiPole
        {
            get
            {
                var node = NejlepsiCesta.GetFirstNode(null);
                return new Tuple<int, int>(node.Item2.X, node.Item2.Y);
            }
        }
        public string Tah
        {
            get
            {
                var dp = DalsiPole;
                if (dp.Item1 > CX) return "P";
                else if (dp.Item1 < CX) return "L";
                else if (dp.Item2 > CY) return "D";
                else if (dp.Item2 < CY) return "N";
                throw new IndexOutOfRangeException();
            }
        }

        public int CX;
        public int CY;

        public Floodfill(int[,] sit, int cx, int cy)
        {
            CX = cx;
            CY = cy;
            Sit = sit;
            //Calculate();
        }

        public void Calculate()
        {
            Cesty.Clear();

            List<FFNode> OpenNodes = new List<FFNode>() { new FFNode(CX, CY, 0) };
            List<FFNode> ClosedNodes = new List<FFNode>();

            while (OpenNodes.Any())
            {
                var current = OpenNodes.OrderBy(o => o.P).First();
                OpenNodes.Remove(current);
                ClosedNodes.Add(current);

                if (Sit[current.Y, current.X] <= 0)
                {
                    var neighbours = MakeNeighbours(current).ToArray();
                    foreach (var item in neighbours)
                    {
                        if (OpenNodes.Contains(item) || ClosedNodes.Contains(item) || Sit[item.Y, item.X] == -1) continue;
                        OpenNodes.Add(item);
                    }
                }
            }

            foreach (var item in ClosedNodes)
            {
                var hodnota = Sit[item.Y, item.X];
                if (hodnota > 0) Cesty.Add(new Tuple<int, FFNode>(hodnota, item));
            }
        }

        public IEnumerable<FFNode> MakeNeighbours(FFNode parent)
        {
            if (parent.X > 0) yield return new FFNode(parent.X - 1, parent.Y, parent.P) { Parent = parent };
            if (parent.X < 9) yield return new FFNode(parent.X + 1, parent.Y, parent.P) { Parent = parent };

            if (parent.Y > 0) yield return new FFNode(parent.X, parent.Y - 1, parent.P) { Parent = parent };
            if (parent.Y < 9) yield return new FFNode(parent.X, parent.Y + 1, parent.P) { Parent = parent };
        }

        public int Heuristic(int startX, int startY, int endX, int endY)
        {
            return (int)(Math.Abs(endX - startX) + Math.Abs(endY - startY));
        }
    }

    public class FFNode
    {
        public int X;
        public int Y;

        public int P;

        public FFNode Parent;

        public FFNode(int x, int y, int p)
        {
            X = x;
            Y = y;
            P = p;
        }

        public Tuple<FFNode, FFNode> GetFirstNode(FFNode parent)
        {
            if (Parent == null) return new Tuple<FFNode, FFNode>(this, parent);
            else return Parent.GetFirstNode(this);
        }

        public override int GetHashCode()
        {
            return ("" + X + Y).GetHashCode();
        }

        public override bool Equals(object obj)
        {
            if (!(obj is FFNode)) return false;
            else
            {
                var n = obj as FFNode;
                return n.X == X && n.Y == Y;
            }
        }
    }
}
