﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LovciPokladu
{
    public partial class Form1 : Form
    {
        Hra AktualniHra;
        public bool AIBezi = false;
        public Random Random = new Random();

        public bool HraAktivni { get { return AktualniHra != null && AktualniHra.Aktivni; } }

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.Focus();
        }

        public void EnableKeyboardControl()
        {
            button2.Text = "zapnout ovladani klavesnici";
            textBox1.Enabled = true;
            richTextBox1.Enabled = true;
            this.Focus();
        }

        public void DisableKeyboardControl()
        {
            button2.Text = "vypnout ovladani klavesnici";
            textBox1.Enabled = false;
            richTextBox1.Enabled = false;
            this.Focus();
        }

        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
            var element = sender as PictureBox;
            var g = e.Graphics;
            var width = element.Width;
            var height = element.Height;

            if (!HraAktivni)
            {
                if(AktualniHra != null){
                    g.Clear(this.BackColor);
                    var str = string.Format("Hra skoncila. Pocet bodu: {0}", AktualniHra.LastResponse.Score);
                    if (AktualniHra.LastResponse.ScoreNepritele != 0)
                    {
                        str += Environment.NewLine + string.Format("Nepritel mel bodu: {0}", AktualniHra.LastResponse.ScoreNepritele);
                        str += Environment.NewLine + (AktualniHra.LastResponse.ScoreNepritele < AktualniHra.LastResponse.Score ? "Vyhra" : "Prohra");
                    }
                    g.DrawString(str, this.Font, new SolidBrush(Color.Black), new PointF(width / 2 - 50, height / 2 - 10));
                }
                return;
            }

            var widthPole = width / AktualniHra.LastResponse.Pole.GetLength(1);
            var heightPole = height / AktualniHra.LastResponse.Pole.GetLength(0);

            Brush volnePoleBrush = new SolidBrush(Color.White);
            Brush zedPoleBrush = new SolidBrush(Color.Black);

            Brush pokladPoleBrush = new SolidBrush(Color.Gold);

            Brush mojeZakladnaPoleBrush = new SolidBrush(Color.DarkGreen);
            Brush zakladnaNepritelePoleBrush = new SolidBrush(Color.DarkRed);

            Brush mujLovecPoleBrush = new SolidBrush(Color.LightGreen);
            Brush lovecNepritelePoleBrush = new SolidBrush(Color.Red);

            for (int y = 0; y < AktualniHra.LastResponse.Pole.GetLength(0); y++)
            {
                for (int x = 0; x < AktualniHra.LastResponse.Pole.GetLength(1); x++)
                {
                    var pole = AktualniHra.LastResponse.Pole[y, x];
                    var rectangle = new Rectangle(x * widthPole, y * heightPole, widthPole, heightPole);

                    if (pole is VolnePole) DrawRect(g, volnePoleBrush, rectangle);
                    else if (pole is ZedPole) DrawRect(g, zedPoleBrush, rectangle);
                    else if (pole is PokladPole) DrawRect(g, pokladPoleBrush, rectangle, (pole as PokladPole).Hodnota.ToString());
                    else if (pole is MojeZakladnaPole) DrawRect(g, mojeZakladnaPoleBrush, rectangle);
                    else if (pole is ZakladnaNepritelePole) DrawRect(g, zakladnaNepritelePoleBrush, rectangle);
                }
            }

            var mujLovecRectangle = new Rectangle(
                (AktualniHra.LastResponse.MujLovec.LovecS - 1) * widthPole,
                (AktualniHra.LastResponse.MujLovec.LovecR - 1) * heightPole,
                widthPole,
                heightPole);

            DrawCirc(g, mujLovecPoleBrush, mujLovecRectangle);


            if (AktualniHra.LastResponse.ViceHracu)
            {
                var nepritelLovecRectangle = new Rectangle(
                    (AktualniHra.LastResponse.LovecNepritele.LovecS - 1) * widthPole,
                    (AktualniHra.LastResponse.LovecNepritele.LovecR - 1) * heightPole,
                    widthPole,
                    heightPole);

                DrawCirc(g, lovecNepritelePoleBrush, nepritelLovecRectangle);
            }
        }

        private void DrawRect(Graphics g, Brush b, Rectangle r, string str = "")
        {
            g.FillRectangle(b, r);
            DrawText(g, r, str);
        }

        private void DrawCirc(Graphics g, Brush b, Rectangle r, string str = "")
        {
            g.FillEllipse(b, r);
            DrawText(g, r, str);
        }

        private void DrawText(Graphics g, Rectangle r, string str = "")
        {
            g.DrawString(str, this.Font, new SolidBrush(Color.Black), r);
        }

        private void Pohni(string where)
        {
            try
            {
                AktualniHra.Tah(where);
                pictureBox1.Invalidate();
                DrawGui();
            }
            catch (InvalidMoveException)
            {

            }
        }

        private void Form1_KeyDown(object sender, KeyEventArgs e)
        {
            if (!HraAktivni) return;

            if (!button1.Enabled)
            {
                if (e.KeyCode == Keys.Left) Pohni("L");
                else if (e.KeyCode == Keys.Right) Pohni("P");
                else if (e.KeyCode == Keys.Up) Pohni("N");
                else if (e.KeyCode == Keys.Down) Pohni("D");

                e.SuppressKeyPress = true;
            }
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            pictureBox1.Invalidate();
        }

        void DrawGui()
        {
            if (!HraAktivni) return;

            label5.Text = AktualniHra.LastResponse.MujLovec.LovecPrineslPoklady.ToString();
            label6.Text = AktualniHra.LastResponse.MujLovec.LovecNesePoklad.ToString();
            label7.Text = AktualniHra.AktualniTah.ToString();
            label8.Text = AktualniHra.ZbyvaTahu.ToString();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            AktualniHra = new Hra(textBox1.Text);
            DrawGui();
            pictureBox1.Invalidate();
            DisableKeyboardControl();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if (button2.Text.Contains("zapnout")) DisableKeyboardControl();
            else EnableKeyboardControl();
        }

        private void Form1_KeyDown(object sender, PreviewKeyDownEventArgs e)
        {

        }

        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            if (!HraAktivni) return false;

            if (!textBox1.Enabled)
            {
                if (keyData == Keys.Left) { Pohni("L"); return true; }
                else if (keyData == Keys.Right) { Pohni("P"); return true; }
                else if (keyData == Keys.Up) { Pohni("N"); return true; }
                else if (keyData == Keys.Down) { Pohni("D"); return true; }
            }
            return false;
        }

        private void button3_Click(object sender, EventArgs e)
        {
            AIBezi = !AIBezi;
            if (AIBezi) button3.Text = "Vypnout AI";
            else button3.Text = "Spustit AI";
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (!AIBezi || !HraAktivni) return;

            string tah;
            //var tah = AktualniHra.MoznePohyby.OrderBy(_ => Random.Next(int.MinValue, int.MaxValue)).First();
            if(AktualniHra.LastResponse.MujLovec.LovecNesePoklad == 0){
                AktualniHra.NavPoklady.Calculate();
                tah = AktualniHra.NavPoklady.Tah;
            }
            else {
                AktualniHra.NavZakladna.Calculate();
                tah = AktualniHra.NavZakladna.Tah;
            }
            Pohni(tah);
        }
    }
}
