﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;

namespace LovciPokladu
{
    class Hra
    {
        public int AktualniTah = 0;
        public ProtocolResponse LastResponse;
        public WebClient Client = new WebClient();
        public string Host;

        public int Id { get { return LastResponse.Id; } }
        public int ZbyvaTahu { get { return LastResponse.ZbyvaTahu; } }
        public bool Aktivni { get { return !LastResponse.Ended; } }

        public int PX { get { return LastResponse.MujLovec.LovecS - 1; } }
        public int PY { get { return LastResponse.MujLovec.LovecR - 1; } }

        public const int W = 10;
        public const int H = 10;

        public List<string> MoznePohyby = new List<string>();

        public Floodfill NavPoklady;
        public Floodfill NavZakladna;

        public Hra(string host)
        {
            Host = host;
            var url = string.Format("http://{0}/start", Host);
            var resp = Client.DownloadString(url);
            LastResponse = new ProtocolResponse(resp);
            CalcMoznePohyby();
            MakeNav();
        }

        public void CalcMoznePohyby()
        {
            MoznePohyby.Clear();
            if (IsTahValid("L")) MoznePohyby.Add("L");
            if (IsTahValid("P")) MoznePohyby.Add("P");
            if (IsTahValid("N")) MoznePohyby.Add("N");
            if (IsTahValid("D")) MoznePohyby.Add("D");
        }

        public void MakeNav()
        {
            MakeNavPoklady();
            MakeNavZakladna();
        }

        public void MakeNavZakladna()
        {
            var sit = new int[10, 10];

            for (int y = 0; y < H; y++)
            {
                for (int x = 0; x < W; x++)
                {
                    var pole = LastResponse.Pole[y, x];
                    if (pole is ZedPole) sit[y, x] = -1;
                    else if (pole is PokladPole) sit[y, x] = 0;
                    else if (pole is MojeZakladnaPole) sit[y, x] = 100;
                    else if (pole is ZakladnaNepritelePole) sit[y, x] = -1;
                    else sit[y, x] = 0;
                }
            }

            NavZakladna = new Floodfill(sit, LastResponse.MujLovec.LovecS - 1, LastResponse.MujLovec.LovecR - 1);
        }

        public void MakeNavPoklady()
        {
            var sit = new int[10, 10];

            for (int y = 0; y < H; y++)
            {
                for (int x = 0; x < W; x++)
                {
                    var pole = LastResponse.Pole[y,x];
                    if (pole is ZedPole) sit[y, x] = -1;
                    else if (pole is PokladPole) sit[y, x] = (pole as PokladPole).Hodnota;
                    else if (pole is MojeZakladnaPole) sit[y, x] = -1;
                    else if (pole is ZakladnaNepritelePole) sit[y, x] = -1;
                    else sit[y, x] = 0;
                }
            }

            NavPoklady = new Floodfill(sit, LastResponse.MujLovec.LovecS - 1, LastResponse.MujLovec.LovecR - 1);
        }

        public ProtocolResponse Tah(string pohyb)
        {
            if (!IsTahValid(pohyb)) throw new InvalidMoveException();

            AktualniTah++;
            var resp = Client.DownloadString(string.Format("http://{0}/tah?id={1}&pohyb={2}", Host, Id, pohyb));
            LastResponse = new ProtocolResponse(resp);
            if (Aktivni)
            {
                CalcMoznePohyby();
                MakeNav();
            }
            return LastResponse;
        }

        public bool IsTahValid(string tah)
        {
            var t = tah.ToLower();

            if (t == "n" && PY < 1) return false;
            if (t == "d" && PY >= H - 1) return false;
            if (t == "l" && PX < 1) return false;
            if (t == "p" && PX >= W - 1) return false;

            if (t == "n" && JeZed(PX, PY - 1)) return false;
            if (t == "d" && JeZed(PX, PY + 1)) return false;
            if (t == "l" && JeZed(PX - 1, PY)) return false;
            if (t == "p" && JeZed(PX + 1, PY)) return false;

            return true;
        }

        public bool JeZed(int x, int y)
        {
            return LastResponse.Zdi[y, x];
        }
    }

    class ProtocolResponse
    {
        public string Response;

        public int Id;
        public int ZbyvaTahu;
        public TypPole[,] Pole = new TypPole[10, 10];

        public Lovec MujLovec;
        public Lovec LovecNepritele;

        public bool ViceHracu { get { return LovecNepritele != null; } }

        public bool Ended = false;
        public int Score = 0;
        public int ScoreNepritele = 0;

        public bool[,] Zdi = new bool[10, 10];

        public ProtocolResponse(string response)
        {
            Response = response;

            var splat = response.Split(new string[] { Environment.NewLine }, StringSplitOptions.None).Where(a => !string.IsNullOrEmpty(a)).ToArray();
            if (splat[0].Contains("Konec hry"))
            {
                var scoreStr = splat[0].Split(':')[1].Split(' ').Where(a => !string.IsNullOrEmpty(a)).ToArray();
                Score = int.Parse(scoreStr[0]);
                if (scoreStr.Length == 2) ScoreNepritele = int.Parse(scoreStr[1]);
                Ended = true;
                return;
            }

            Id = int.Parse(splat[0]);
            ZbyvaTahu = int.Parse(splat[1]);

            for (int y = 0; y < 10; y++)
            {
                for (int x = 0; x < 10; x++)
                {
                    string typ = "" + splat[2 + y][x * 2] + splat[2 + y][x * 2 + 1];
                    Pole[y, x] = ParsePole(typ.ToLower());

                    Zdi[y, x] = Pole[y, x] is ZedPole || Pole[y, x] is ZakladnaNepritelePole;
                }
            }

            MujLovec = new Lovec(splat[12]);
            if (splat.Length == 14) LovecNepritele = new Lovec(splat[13]);
        }

        private static TypPole ParsePole(string pole)
        {
            int poklad;
            if (int.TryParse(pole, out poklad)) return new PokladPole(poklad);

            switch (pole)
            {
                case "..": return new VolnePole();
                case "xx": return new ZedPole();
                case "++": return new MojeZakladnaPole();
                case "--": return new ZakladnaNepritelePole();
                default: throw new IndexOutOfRangeException();
            }
        }
    }

    public class Lovec
    {
        public int LovecS;
        public int LovecR;

        public int LovecNesePoklad;
        public int LovecPrineslPoklady;

        public Lovec(string str)
        {
            var lovec = str.Split(' ').Select(a => int.Parse(a)).ToArray();
            LovecS = lovec[0];
            LovecR = lovec[1];

            LovecNesePoklad = lovec[2];
            LovecPrineslPoklady = lovec[3];
        }
    }

    public abstract class TypPole
    {
    }

    public class VolnePole : TypPole { }
    public class ZedPole : TypPole { }
    public class MojeZakladnaPole : TypPole { }
    public class ZakladnaNepritelePole : TypPole { }

    public class PokladPole : TypPole
    {
        public int Hodnota;

        public PokladPole(int hodnota)
        {
            Hodnota = hodnota;
        }
    }

    [Serializable]
    public class InvalidMoveException : Exception
    {
        public InvalidMoveException() { }
        public InvalidMoveException(string message) : base(message) { }
        public InvalidMoveException(string message, Exception inner) : base(message, inner) { }
        protected InvalidMoveException(
          System.Runtime.Serialization.SerializationInfo info,
          System.Runtime.Serialization.StreamingContext context)
            : base(info, context) { }
    }
}
